import re
import subprocess
from dataclasses import dataclass
from typing import Optional

import wmi

from mag_tools.bean.sys.operate_system import OperateSystem
from mag_tools.model.cpu_series import CpuSeries


@dataclass
class Cpu:
    id: Optional[str] = None    # 标识
    serial_number: Optional[str] = None            # 序列号
    name: Optional[str] = None            # 名称
    cpu_type: Optional[str] = None        # CPU类型
    cpu_series: Optional[CpuSeries] = None # CPU系列
    cores: Optional[int] = None           # 核心数量
    performance_cores: Optional[int] = None # 性能核心数量，可选
    efficiency_cores: Optional[int] = None # 能效核心数量，可选
    threads: Optional[int] = None         # 线程数量，可选
    base_clock: Optional[float] = None    # 基础频率（单位：GHz），可选
    boost_clock: Optional[float] = None   # 最大睿频（单位：GHz），可选
    cache: Optional[int] = None           # 缓存大小（单位：MB），可选
    tdp: Optional[int] = None             # 热设计功耗（单位：W），可选
    process: Optional[int] = None         # 制造工艺（单位：nm），可选
    architecture: Optional[str] = None    # 架构类型，可选
    model: Optional[str] = None           # 型号

    @classmethod
    def get_info(cls):
        """
        获取当前系统的CPU信息，并返回一个Cpu实例
        """
        if OperateSystem.is_windows():
            return cls.__get_from_windows()
        elif OperateSystem.is_linux():
            return cls.__get_from_linux()
        return None

    @classmethod
    def __get_from_windows(cls):
        """
        从 Windows 系统获取 CPU 信息
        """
        c = wmi.WMI()
        cpu_info = c.Win32_Processor()[0]

        cpu_series, model = cls.__parse_brand_raw(cpu_info.Name.strip())

        return cls(
            serial_number=cpu_info.ProcessorId.strip(),
            name=cpu_info.Name.strip(),
            cpu_type=cpu_info.ProcessorType,
            cpu_series=cpu_series,
            base_clock=float(cpu_info.MaxClockSpeed) / 1000,  # 转换为 GHz
            boost_clock=float(cpu_info.MaxClockSpeed) / 1000,  # 转换为 GHz
            cores=cpu_info.NumberOfCores,
            performance_cores=cpu_info.NumberOfCores,  # 假设性能核心数等于核心数
            efficiency_cores=(cpu_info.ThreadCount - cpu_info.NumberOfCores) if cpu_info.ThreadCount and cpu_info.NumberOfCores else None,
            threads=cpu_info.ThreadCount,
            architecture=cpu_info.Architecture,
            cache=cpu_info.L2CacheSize,
            model=model,
        )

    @classmethod
    def __get_from_linux(cls):
        """
        从 Linux 系统获取 CPU 信息
        """
        try:
            result = subprocess.run(["lscpu"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, check=True)
            info = {}
            for line in result.stdout.split("\n"):
                if "Model name:" in line:
                    info["name"] = line.split(":")[1].strip()
                elif "Architecture:" in line:
                    info["architecture"] = line.split(":")[1].strip()
                elif "CPU(s):" in line:
                    info["threads"] = int(line.split(":")[1].strip())
                elif "Core(s) per socket:" in line:
                    info["cores"] = int(line.split(":")[1].strip())
                elif "CPU MHz:" in line:
                    info["base_clock"] = float(line.split(":")[1].strip()) / 1000  # 转换为 GHz
                elif "BogoMIPS:" in line:
                    info["boost_clock"] = float(line.split(":")[1].strip()) / 1000  # 转换为 GHz
                elif "L2 cache:" in line:
                    info["cache"] = int(line.split(":")[1].strip().replace("K", ""))  # 转换为 KB
                elif "Vendor ID:" in line:
                    info["cpu_type"] = line.split(":")[1].strip()
                elif "Model:" in line:
                    info["model"] = line.split(":")[1].strip()

            cpu_series, model = cls.__parse_brand_raw(info.get("name"))

            return cls(
                name=info.get("name"),
                cpu_type=info.get("cpu_type"),
                cpu_series=cpu_series,
                base_clock=info.get("base_clock"),
                boost_clock=info.get("boost_clock"),
                cores=info.get("cores"),
                performance_cores=info.get("cores"),  # 假设性能核心数等于核心数
                efficiency_cores=(info.get("threads") - info.get("cores")) if info.get("threads") is not None and info.get("cores") is not None else None,
                threads=info.get("threads"),
                architecture=info.get("architecture"),
                cache=info.get("cache"),
                model=model,
            )
        except subprocess.CalledProcessError as e:
            print(f"Error getting CPU information: {e}")
            return cls()

    def __str__(self):
        """
        返回CPU参数的字符串表示
        """
        attributes = [f"Type: {self.cpu_type}"]
        attributes += [f"{attr.replace('_', ' ').title()}: {getattr(self, attr)}" for attr in vars(self) if getattr(self, attr) is not None and attr not in ['id', 'model']]
        return ", ".join(attributes)

    @classmethod
    def __parse_brand_raw(cls, brand_raw: Optional[str] = None):
        """
        从字符串中提取CPU参数并创建CPUParameters实例
        """
        pattern = r"(?P<generation>\d+th Gen)?\s*(?P<brand>Intel\(R\) Core\(TM\)|AMD Ryzen)\s*(?P<series>i\d|Ryzen \d{3,4}X?)-?(?P<model>\d+K?)?"

        match = re.match(pattern, brand_raw)
        if match:
            cpu_series = CpuSeries.of_code(match.group('series'))
            model = match.group("model")
            return cpu_series, model
        return None, None

if __name__ == '__main__':
    cpu = Cpu.get_info()
    print(cpu)



from typing import Any, List

import numpy as np
from mag_tools.utils.data.string_format import StringFormat
from mag_tools.utils.data.string_utils import StringUtils

from mag_tools.bean.text_format import TextFormat
from mag_tools.model.data_type import DataType


class ArrayUtils:
    @staticmethod
    def text_to_array_1d(text: str, data_type: DataType) -> List[Any]:
        """
        从一行文本中获取数值数组[]
        :param text: 文本，由 "数值" 或 "数值个数*数值"构成
        :param data_type: 数据类型
        :return: 数值数组
        """
        data = []

        for part in text.split():
            if '*' in part:
                count, value = part.split('*')
                data.extend([StringUtils.to_value(value, data_type)] * int(count))
            else:
                data.append(StringUtils.to_value(part, data_type))

        return data

    @staticmethod
    def lines_to_array_1d(lines: List[str], data_type: DataType) -> List[Any]:
        """
        从多行文本中获取数值数组[]
        :param lines: 文本段，每行由 "数值" 或 "数值个数*数值"构成
        :param data_type: 数据类型
        :return: 数值数组
        """
        array_1d = []
        for line in lines:
            array_1d.extend(ArrayUtils.text_to_array_1d(line, data_type))
        return array_1d

    @staticmethod
    def lines_to_array_2d(lines: List[str], nx: int, ny: int, data_type: DataType) -> List[
        List[Any]]:
        """
        从文本块获取数值列表
        :param lines: 文本块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        :param nx: 行数
        :param ny: 列数
        :param data_type: 数据类型
        :return: Values
        """
        array_1d = ArrayUtils.lines_to_array_1d(lines, data_type)
        return np.array(array_1d).reshape((nx, ny)).tolist()

    @staticmethod
    def lines_to_array_3d(block_lines: List[str], nx: int, ny: int, nz: int, data_type: DataType) -> List[ List[List[Any]]]:
        """
        从文本块获取数值列表
        :param block_lines: 文本块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        :param nx: 行数
        :param ny: 列数
        :param nz: 层数
        :param data_type: 数据类型
        :return: Values
        """
        array_1d = ArrayUtils.lines_to_array_1d(block_lines, data_type)
        return np.array(array_1d).reshape((nx, ny, nz)).tolist()

    @staticmethod
    def array_1d_to_lines(array_1d: List[Any], text_format: TextFormat) -> List[str]:
        """
        将一维数组转换为多行文本表示
        :param array_1d: 一维数组，包含任意类型的数值
        :param text_format: 文本格式，包含每行数值个数、对齐方式和行首标识符等信息
        :return: 多行文本表示的数组
        """
        if array_1d is None:
            return []

        result = []
        count = 1
        current_value = array_1d[0]

        max_len = 2
        for i in range(1, len(array_1d)):
            if array_1d[i] == current_value:
                count += 1
            else:
                current_text = StringFormat.format_number(current_value, text_format.get_data_format(current_value))
                txt = f"{count}*{current_text}" if count > 1 else f"{current_text}"
                max_len = max(max_len, len(txt))
                result.append(txt)

                current_value = array_1d[i]
                count = 1

        # 处理最后一组数据
        current_text = StringFormat.format_number(current_value, text_format.get_data_format(current_value))
        txt = f"{count}*{current_text}" if count > 1 else f"{current_text}"
        max_len = max(max_len, len(txt))
        result.append(txt)

        # 将结果按行分隔，每行包含data_format.number_per_line个数值
        lines = []
        for i in range(0, len(result), text_format.number_per_line):
            txt = StringFormat.pad_values(result[i:i + text_format.number_per_line], max_len, text_format.justify_type)
            lines.append(f'{text_format.at_header}{txt}')

        return lines

    @staticmethod
    def array_2d_to_lines(array_2d: List[List[Any]], text_format: TextFormat) -> List[str]:
        """
        生成数值块
        :param array_2d: 二维数组
        :param text_format: 数据格式
        :return: 数值块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        """

        # 将其展平为一维数组后添加到列表中
        layer = np.array(array_2d).flatten()
        return ArrayUtils.array_1d_to_lines(layer.tolist(), text_format)

    @staticmethod
    def array_3d_to_lines(array_3d: List[List[List[Any]]], text_format: TextFormat) -> List[str]:
        """
        生成数值块
        :param array_3d: 三维数组
        :param text_format: 数据格式
        :return: 数值块，由多行构成，每行由 "数值" 或 "数值个数*数值"构成
        """

        # 遍历每一层并将其展平为一维数组后添加到列表中
        array_1d_sum = []
        num_layers = np.array(array_3d).shape[0]  # 层数
        for i in range(num_layers):
            array_2d = np.array(array_3d[i]).flatten()
            array_1d = np.array(array_2d).flatten()
            array_1d_sum.extend(array_1d)
            # layer_lines = ValueUtils.array_2d_to_lines(layer.tolist(), text_format)
        return ArrayUtils.array_1d_to_lines(array_1d_sum, text_format)


if __name__ == '__main__':
    _lines = ['60*0.087', '60*0.097']
    # _lines = ['1.18379        2.26189        2.61705        5.99524       10.31299',
    #           '15.17937       32.40075       33.90037      109.47345      261.72995',
    #           '238.24753      206.15742',
    #           '238.24753      206.15742      121.33305      242.47713      166.94165',
    #           '146.50681       13.91259        7.39986        5.83849        6.06996',
    #           '7.95473        8.99119']
    dt = ArrayUtils.lines_to_array_3d(_lines, 20, 3, 2, DataType.FLOAT)

    _text_format = TextFormat(number_per_line=5, decimal_places_of_zero=3, decimal_places=3)
    te = ArrayUtils.array_3d_to_lines(dt, _text_format)
    print("\n".join(te))



from mag_tools.model.base_enum import BaseEnum


class LogType(BaseEnum):
    ROOT = ('root', '根')
    FRAME = ('frame', '框架')
    DAO = ('dao', '数据库')
    SERVICE = ('service', '服务')
    PERFORMANCE = ('performance', '性能')



import hashlib
from typing import Union

from exception.app_exception import AppException
from mag_tools.utils.data.bytes_utils import BytesUtils
from model.digest_alg import DigestAlg


class Digest:
    @staticmethod
    def digest_bytes(message: bytes, algorithm: DigestAlg) -> bytes:
        """
        计算摘要
        :param message: 消息内容
        :param algorithm: 摘要算法
        :return: 摘要
        """
        try:
            digest = hashlib.new(algorithm.code.lower())
            digest.update(message)
            ba_hash = digest.digest()
            return ba_hash
        except ValueError as e:
            raise AppException(f"该摘要算法[{algorithm}]不存在") from e

    @staticmethod
    def digest(message: str, algorithm: DigestAlg) -> Union[str, bytes]:
        """
        计算摘要
        :param message: 消息内容
        :param algorithm: 摘要算法
        :return: 摘要(十六进制格式) 或 摘要
        """
        ba_message = message.encode()
        ba_hash = Digest.digest_bytes(ba_message, algorithm)
        return BytesUtils.bytes_to_hex(ba_hash)

    @staticmethod
    def sha256(message: str) -> str:
        """
        计算SHA256摘要
        :param message: 消息内容
        :return: 摘要(十六进制格式)
        """
        return Digest.digest(message, DigestAlg.SHA256)

    @staticmethod
    def sha512(message: str) -> str:
        """
        计算SHA512摘要
        :param message: 消息内容
        :return: 摘要(十六进制格式)
        """
        return Digest.digest(message, DigestAlg.SHA512)

    @staticmethod
    def sha3_256(message: str) -> str:
        """
        计算SHA3-256摘要
        :param message: 消息内容
        :return: 摘要(十六进制格式)
        """
        return Digest.digest(message, DigestAlg.SHA3_256)

    @staticmethod
    def sha3_512(message: str) -> str:
        """
        计算SHA3-512摘要
        :param message: 消息内容
        :return: 摘要(十六进制格式)
        """
        return Digest.digest(message, DigestAlg.SHA3_512)

    @staticmethod
    def digest_more_times_bytes(message: bytes, algorithm: DigestAlg, hash_times: int) -> bytes:
        """
        进行多次Hash
        :param message: 消息内容
        :param algorithm: 摘要算法
        :param hash_times: hash次数
        :return: 摘要
        """
        ba_hash = message
        for _ in range(hash_times):
            ba_hash = Digest.digest(ba_hash, algorithm)
        return ba_hash

    @staticmethod
    def digest_more_times(message: str, algorithm: DigestAlg, hash_times: int) -> str:
        """
        进行多次Hash
        :param message: 消息内容
        :param algorithm: 摘要算法
        :param hash_times: hash次数
        :return: 摘要
        """
        ba_hash = Digest.digest_more_times_bytes(message.encode(), algorithm, hash_times)
        return BytesUtils.bytes_to_hex(ba_hash)
